#!/usr/bin/python
# -*- coding: iso-8859-1 -*-

##############################################################################################
# Copyright (c) by George Ruinelli 
# License: 	GPL3
# Description:	Basic GUI for Syncevolution
##############################################################################################

# USER CONFIGURATION #################
peer = "" #peer to be synced, keep empty to use default peer configured in /home/user/.config/syncevolution/config.ini
sources = "calendar" #default sources to be synced
syncmode = "two-way" #default sync mode

autostart_timeout = 5 #seconds

######################################

version = "0.3"
build = "1"



import sys
from PySide.QtCore import *
from PySide.QtGui import *
from PySide.QtDeclarative import *
from PySide.QtGui import QDesktopServices as QDS

import math
import string
import platform
import os

import subprocess
import time

import threading
import signal


class data:
  myThread = None
  
    
####################################################################################################

class QML_to_Python_Interface(QObject):
    @Slot(str, str)
    def start(self, syncmode, sources):
	print "start"
	#return
	data.myThread =MyThread(peer, sources, syncmode)
	data.myThread.setSyncmode(syncmode, sources) 
	if(data.myThread.getStatus() == False):
	    data.myThread.start()  

	
    @Slot(result=str)
    def getVersion(self):
      return version +"."+build
	
    @Slot(result=str)
    def getSyncEvolutionVersion(self):
	cmd = "syncevolution --version"
	output = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE).communicate()[0]	    
	#print output
	output = output.split("\n")
	return output[0].replace("SyncEvolution", "")
	
	    
    
    @Slot(result=str)
    def update(self):
	#print "Python update"
	txt = data.myThread.getOutput()
	#txt ="."
	#print txt
	#print "> done, return", len(txt), "bytes"
	return txt
	
    
    @Slot()
    def terminate(self):
	print "terminate"
	data.myThread.proc.kill()
	data.myThread.stop()
	
	
    
    @Slot(result=bool)
    def status(self):
	return data.myThread.getStatus()
	
    
    @Slot(result=float)
    def getRunTime(self):
	return int(data.myThread.getRunTime() * 10) / 10.0
    
    
    #@Slot()
    #def openLogInBrowser(self):      
	#new = 0 # open in a new tab, if possible
	## open an HTML file on my own (Windows) computer
	#url = "/home/user/.cache/syncevolution/target_+config@owncloud-2012-10-28-15-13-a/syncevolution-log.html"
	#print "Open log in browser:", url
	#cmd = "/usr/bin/grob " + url
	#output = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE).communicate()[0]	
    
    
################################################################################################  
    








class MyThread(threading.Thread):
    def __init__(self, peer, sources, syncmode): 
	threading.Thread.__init__(self) 		
	print "init myThread"
	self.proc = ""
	self.proc_is_running = False
	self.output = ""
	self.syncmode = syncmode
	self.peer = peer
	self.sources = sources
	self.time = time.time()
	
	self.timer = QTimer()
	QObject.connect(self.timer, SIGNAL("timeout()"), self.update)
	
	if(self.peer == ""):
	    try:
		f = open('/home/user/.config/syncevolution/config.ini', 'r')
		data = f.readline()
		data = f.readline()
		arr = data.split("=")
		f.close()
		self.peer = arr[1][:-1]
		self.peer = self.peer.strip()
		#print "Peer: " + self.peer
	    except:
	      print "Peer empty and could not open config file."
	      quit()
	    
	
    def setSyncmode(self, syncmode, sources):
	self.syncmode = syncmode
	self.sources = sources


    def run(self):
	print "Start myThread"
	if(self.proc_is_running == True): 
	    print "myThread already running"
	    return
	self.proc_is_running = True
	
	self.output = "Peer: " + self.peer + ", sources: " + self.sources + ", mode: " + self.syncmode + "\n"
	
	
	try:
	    command = "sh ./syncevolution.sh " +  str(self.syncmode) + " " + self.peer + " " + self.sources
	    #self.output  = command + "\n"
	    print "Command:", command
	    self.proc = subprocess.Popen(command,
			      shell=True,
			      stdout=subprocess.PIPE,
			      preexec_fn=os.setsid
			  )
	    self.timer.start(10)
	    
	except:
	    print "could not start myThread"

	

    def update(self):
	#print "################ UPDATE ##############################"
	r = self.proc.stdout.readline()
	#print "   r: ", r
	#print r[:7]
	if(r == "....\n"): #prevention of response timeout, do not show in output
	    pass
	elif(r == "::::\n"): #syncevolution is done, kill all processes
	    self.output = self.output + r
	    self.stop()
	#elif(r[:7] == "[ERROR]" and r[:36] != "[ERROR] stderr: extendedstorage.cpp"): #syncevolution has an error, kill all processes
	    #t = "Received error, synchronisation stopped!"
	    #print r
	    #self.output = self.output + r + t
	    #print r
	    #self.stop()
	else: #normal output
	    self.output = self.output + r
	    print r
	  
	#print "self.proc.poll:", self.proc.poll()
	if(self.proc.poll() != None):
	    print "Process exited"
	    if(self.output == ""): self.stop()
	    
    def getRunTime(self):
	return time.time() - self.time


    def stop(self):
	print "stop myThread"
	if(self.proc_is_running == False):
	    print "myThread not running"
	    return
	self.proc_is_running = False
	try:
	    os.killpg(self.proc.pid, signal.SIGTERM)  # Send the signal to all the process groups
	except:
	    print "could not stop myThread"
	try:
	    os.popen("killall -9 'syncevolution'") # Send the signal to all syncevolution processes
	except:
	    print "could not stop syncevolution processes. Maybe none are running?"
	self.timer.stop()
	
	#self._stop.set()
	#self.join()


    def stopped(self):
        return self._stop.isSet()
      
      


    def getOutput(self):
	txt = self.output
	self.output = ""
	return txt



    def getStatus(self):
	return self.proc_is_running









    
################################################################################################  



if __name__ == '__main__':
    print "Starting Syncevolution-GUI"
   
    app = QApplication(sys.argv)

    view = QDeclarativeView()
    view.setSource(QUrl.fromLocalFile('qml/main.qml'))
    root = view.rootObject()
    
    root.setTimeout(autostart_timeout)


    # instantiate the Python object
    qml_to_python = QML_to_Python_Interface()
    # expose the object to QML
    context = view.rootContext()
    context.setContextProperty("qml_to_python", qml_to_python)
    
    
    
    
    
    
    
    if(platform.machine().startswith('arm')): view.showFullScreen()
    view.show()
    
    
    #data.myThread = MyThread(peer, sources, syncmode)
        
    app.exec_() #endless loop

    #data.myThread.stop()
    
    
    
    
    
    